from django import forms
from django.templatetags.static import static


class BaseEditorWidget(forms.Textarea):
    """
    Base editor widget.
    """

    class Media:
        """
        Include JavaScripts and CSS here for CodeMirror, modes, and themes, along with
        the initialization script.
        """

        css = {
            "all": [
                "https://cdnjs.cloudflare.com/ajax/libs/codemirror/5.48.4/codemirror.min.css",
                "https://cdnjs.cloudflare.com/ajax/libs/codemirror/5.48.4/theme/base16-light.min.css",
                "https://cdnjs.cloudflare.com/ajax/libs/codemirror/5.48.4/theme/base16-dark.min.css",
                "https://cdnjs.cloudflare.com/ajax/libs/codemirror/5.48.4/theme/solarized.min.css",
            ]
        }
        js = [
            "https://cdnjs.cloudflare.com/ajax/libs/codemirror/5.48.4/codemirror.min.js",
            "https://cdnjs.cloudflare.com/ajax/libs/codemirror/5.48.4/addon/mode/overlay.min.js",
            "https://cdnjs.cloudflare.com/ajax/libs/codemirror/5.48.4/mode/xml/xml.min.js",
            "https://cdnjs.cloudflare.com/ajax/libs/codemirror/5.48.4/mode/htmlmixed/htmlmixed.min.js",
            "https://cdnjs.cloudflare.com/ajax/libs/codemirror/5.48.4/mode/django/django.min.js",
            "https://cdnjs.cloudflare.com/ajax/libs/codemirror/5.48.4/mode/css/css.min.js",
            "https://cdnjs.cloudflare.com/ajax/libs/codemirror/5.48.4/mode/sass/sass.min.js",
            "https://cdnjs.cloudflare.com/ajax/libs/codemirror/5.48.4/mode/javascript/javascript.min.js",
            static("impression/js/codemirror-init.js"),
        ]

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        # add class to flag this widget as an editor
        self.attrs["class"] = "impression-template-editor"


class TemplateEditorWidget(BaseEditorWidget):
    """
    Editor widget for Django templates (HTML/Django/CSS/JS).
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        # set language mode
        self.attrs["class"] += " impression-mode-django"


class AutogeneratePlainTextWidget(forms.CheckboxInput):
    """
    Custom checkbox that hides the plaintext body when checked and reveals it when
    unchecked.
    """

    class Media:
        js = [static("impression/js/autogenerated-field-hider.js")]
